import time
from typing import List

import pygame
from pygame import Surface

from controllers.ControllerGame import ControllerGame
from controllers.ControllerTank import ControllerTank
from models.Game import Game
from models.enums.EnumGameObjectDirection import EnumGameObjectDirection
from models.enums.EnumGameObjectType import EnumGameObjectType
from views.components.ComponentGameObject import ComponentGameObject, sprite_width
from views.components.ComponentTank import ComponentTank
from views.components.base.ComponentBase import ComponentBase
from views.components.ui.ComponentButton import ComponentButton
from views.windows.WindowGame import WindowGame
from views.windows.WindowHighScores import WindowHighScores, ReactPropsWindowHighScores


class WindowMenu:
    def __init__(self):
        super().__init__()

        self.screen = pygame.display.set_mode(
            (640, 640)
        )
        self.is_window_visible = True
        self.ui_components: List[ComponentBase] = []

        button_start = ComponentButton(
            rect=pygame.Rect(250, 200, 150, 40),
            text="Start game",
        )
        self.ui_components.append(button_start)
        button_start.add_listener_click(self.on_click_start_game)

        button_load = ComponentButton(
            rect=pygame.Rect(250, 250, 150, 40),
            text="Load game",
        )
        self.ui_components.append(button_load)
        button_load.add_listener_click(self.on_click_load_game)

        button_save = ComponentButton(
            rect=pygame.Rect(250, 300, 150, 40),
            text="Save game",
        )
        self.ui_components.append(button_save)
        button_save.add_listener_click(self.on_click_save_game)

        button_exit = ComponentButton(
            rect=pygame.Rect(250, 350, 150, 40),
            text="Quit",
        )
        self.ui_components.append(button_exit)
        button_exit.add_listener_click(self.on_click_quit_game)

    def on_click_start_game(self):
        window_main = WindowGame()
        window_main.show()

        window_highscores = WindowHighScores(
            props=ReactPropsWindowHighScores(
                x=0,
                y=0,
                width=640,
                height=640,
                current_score=ControllerGame.instance().game.score
            )
        )
        window_highscores.show(self.screen)

    def on_click_load_game(self):
        with open("game.json", "r") as file:
            game_json = file.read()
            game = Game.from_json(game_json)
            ControllerGame.instance().set_game(game)
        print("Game loaded from game.json")

        window_main = WindowGame()
        window_main.show()

    def on_click_save_game(self):
        game_json = ControllerGame.instance().game.to_json()
        with open("game.json", "w") as file:
            file.write(game_json)
        print("Game saved to game.json")

    def on_click_quit_game(self):
        self.is_window_visible = False

    def show(self):
        # main game loop
        time_last = pygame.time.get_ticks()
        while self.is_window_visible:
            # get delta seconds
            time_current = pygame.time.get_ticks()
            delta_milisec = time_current - time_last
            time_last = time_current

            # update
            self.update(delta_milisec)

            # draw
            self.render()

            # update display
            pygame.display.flip()
            time.sleep(0.1)

    def update(self, delta_milisec):
        for ui_components in self.ui_components:
            ui_components.update(delta_milisec)

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.is_window_visible = False
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    self.is_window_visible = False
            elif event.type == pygame.MOUSEMOTION:
                for ui_components in self.ui_components:
                    ui_components.on_mouse_move(event.pos)
            elif event.type == pygame.MOUSEBUTTONDOWN:
                for ui_components in self.ui_components:
                    ui_components.on_mouse_button_down()
            elif event.type == pygame.MOUSEBUTTONUP:
                for ui_components in self.ui_components:
                    ui_components.on_mouse_button_up()

    def render(self):
        self.screen.fill((0, 0, 0))
        for ui_components in self.ui_components:
            ui_components.render(self.screen)